from corpus import Corpus
from underthesea.corpus.document import Document
from os.path import join
from os import listdir, mkdir

from underthesea.transformer.unicode import UnicodeTransformer


class PlainTextCorpus(Corpus):
    """class for handling plain text corpus
    """

    def __init__(self):
        self.documents = None

    def load(self, folder):
        """load plaintext folder to documents and apply unicode transformer

        :param str folder: path to directory
        :type folder: string
        """
        ids = listdir(folder)
        files = [join(folder, f) for f in ids]
        contents = [open(f, "r").read() for f in files]
        documents = []

        for id, content in zip(ids, contents):
            document = Document(id)
            sentences = content.split("\n")
            unicode_transformer = UnicodeTransformer()
            sentences = [unicode_transformer.transform(sentence) for sentence in sentences]
            document.set_sentences(sentences)
            documents.append(document)
        self.documents = documents

    def save(self, folder):
        """save corpus to files

        :param str folder: path to directory
        :type folder: string
        """
        try:
            mkdir(folder)
        except Exception as e:
            pass
        for document in self.documents:
            f = join(folder, document.id)
            content = u"\n".join(document.sentences)
            content = content.encode("utf-8")
            open(f, "w").write(content)



import re


def tokenize(text):
    """
    pre-process text for word segment

    :param text: raw text input
    :return: tokenize text
    """
    specials = ["==>", "->", "\.\.\.", ">>"]
    digit = "\d+([\.,_]\d+)+"
    email = "(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)"
    web = "^(http[s]?://)?(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+$"
    word = "\w+"
    non_word = "[^\w\s]"

    patterns = []
    patterns.extend(specials)
    patterns.extend([web, email])
    patterns.extend([digit, non_word, word])

    patterns = "(" + "|".join(patterns) + ")"
    tokens = re.findall(patterns, text, re.UNICODE)
    return u" ".join(["%s" % token[0] for token in tokens])



from os.path import dirname, join
from unittest import TestCase
from os.path import join
from underthesea.corpus.ws import WSCorpus



class TestWSCorpus(TestCase):

    def setUp(self):
        data_folder = dirname(__file__)
        self.folder = join(data_folder, "sample_ws_corpus")
        self.saved_ws_folder = join(data_folder, "sample_saved_ws_corpus")

    def test_load_column_format(self):
        corpus = WSCorpus()
        corpus.load(self.folder, "COLUMN")
        self.assertEqual(1561, len(corpus.documents[0].sentences))
        self.assertGreater(len(corpus.documents[0].sentences[0]), 0)

    def test_save_text_format(self):
        corpus = WSCorpus()
        corpus.load(self.folder, "COLUMN")
        corpus.save(self.saved_ws_folder, "TEXT")

    def test_save_text_format(self):
        corpus = WSCorpus()
        corpus.load(self.folder, "COLUMN")
        corpus.save(self.saved_ws_folder, "TEXT")



from unittest import TestCase, skip
from underthesea.corpus import viet_dict_11K, viet_dict_22K, viet_dict_39K, viet_dict_74K


class Test_VietDict(TestCase):
    def test_viet_dict(self):
        words = viet_dict_11K.words
        self.assertEqual(11373, len(words))

    def test_unicode(self):
        word = viet_dict_11K.words[0]
        self.assertEqual(unicode, type(word))

    @skip("")
    def test_viet_dict_22K(self):
        words = viet_dict_22K.words
        self.assertEqual(22426, len(words))

    @skip("")
    def test_viet_dict_39K(self):
        words = viet_dict_39K.words
        self.assertEqual(39071, len(words))

    @skip("")
    def test_viet_dict_74K(self):
        words = viet_dict_74K.words
        self.assertEqual(73901, len(words))



from os.path import dirname, join

import pycrfsuite

from underthesea.word_sent.transformer import Transformer


class CRFModel:
    def __init__(self):
        """
        load trained data from crf-model
        """
        self.model = pycrfsuite.Tagger()
        filepath = join(dirname(__file__), "crf-model")
        self.model.open(filepath)
        filepath = join(dirname(__file__), "punctuation.txt")
        self.punctuation = open(filepath, "r").read().split("\n")

    def predict(self, sentence):
        """

        :param unicode|str sentence: raw sentence
        :return: segmented sentence
        :rtype: unicode|str
        """
        sentence = Transformer.transform(sentence)
        tags = self.model.tag(sentence)
        tokenized_sentence = u''
        for tag, word in zip(tags, sentence):
            word = word[0]
            if tag == "I_W":
                tokenized_sentence = tokenized_sentence + u"_" + word
            else:
                tokenized_sentence = tokenized_sentence + word
            tokenized_sentence += " "
        format_sentence = ''
        for word in tokenized_sentence.split("_"):
            if word not in self.punctuation:
                format_sentence += word[: -1] + "_"
        tokenized_sentence = format_sentence[:-1]
        return tokenized_sentence



# -*- coding: utf-8 -*-


from underthesea.word_sent.model import CRFModel
from tokenize import tokenize


def word_sent(sentence):
    """
    :param unicode|str sentence: raw sentence
    :return: segmented sentence
    :rtype: unicode|str
    """
    sentence = tokenize(sentence)
    crf_model = CRFModel()
    return crf_model.predict(sentence)



class LowercaseTransformer:
    def __init__(self):
        pass

    def transform(self, text):
        """
        :param unicode|str text: input text
        :type text: unicode|str

        :return: transformed text
        :rtype: unicode|str
        """
        return text.lower()



from underthesea.corpus.readers.dictionary_loader import DictionaryLoader

words = DictionaryLoader('Viet74K.txt').words


def word2features(sent, i):
    """
    add feature for each word

    :param unicode|str sent: input sentence
    :param int i: index of word in sentence
    :return: word added feature
    :rtype: list
    """
    word = sent[i][0]
    features = [
        word,
        'bias',
        'word.lower=' + word.lower(),
        'word.isupper=%s' % word.isupper(),
        'word.istitle=%s' % word.istitle(),
        'word.isdigit=%s' % word.isdigit(),
    ]
    if i >= 1:
        word1 = sent[i - 1][0]
        features.extend([
            '-1:word.lower=' + word1.lower(),
            '-1:word.istitle=%s' % word1.istitle(),
            '-1:word.isupper=%s' % word1.isupper(),
        ])
    else:
        features.append('BOS')

    if i < len(sent) - 1:
        word = " ".join([sent[i][0].lower(), sent[i + 1][0].lower()])
        features.extend([
            '2_word.in_dictionary=' + str(int(word in words)),
        ])
    else:
        features.append('2_word:EOS')

    if i >= 1:
        word = " ".join([sent[i - 1][0].lower(), sent[i][0].lower()])
        features.extend([
            '-2_word.in_dictionary=' + str(int(word in words)),
        ])
    else:
        features.append('-2_word:BOS')

    if i >= 2:
        word = " ".join([sent[i - 2][0].lower(), sent[i - 1][0].lower(), sent[i][0].lower()])
        features.extend([
            '-3_word.in_dictionary=' + str(int(word in words)),
        ])
    else:
        features.append('-3_word:BOS')

    if i < len(sent) - 2:
        word = " ".join([sent[i][0].lower(), sent[i + 1][0].lower(), sent[i + 2][0]])
        features.extend([
            '3_word.in_dictionary=' + str(int(word in words)),
        ])
    else:
        features.append('3_word:EOS')

    if i >= 2:
        word2 = sent[i - 2][0]
        features.extend(['-2:word.lower=' + word2.lower()])
    else:
        features.append('BOS')

    if i < len(sent) - 1:
        next_word_1 = sent[i + 1][0]
        features.extend([
            '+1:word.lower=' + next_word_1.lower()
        ])
    else:
        features.append('EOS')

    if i < len(sent) - 2:
        next_word_2 = sent[i + 2][0]
        features.extend([
            '+2:word.lower=' + next_word_2.lower()
        ])
    else:
        features.append('EOS')

    return features



from os.path import join, dirname
from underthesea.transformer.unicode import UnicodeTransformer


class DictionaryLoader:
    def __init__(self, filepath):
        """load words from Ho Ngoc Duc's dictionary

        :param str filepath: filename of dictionary data
        :type filepath: str
        """
        data_folder = join(dirname(dirname(__file__)), "data")
        data_file = join(data_folder, filepath)
        self.data_file = data_file
        self.words_data = None

    @property
    def words(self):
        if not self.words_data:
            words = open(self.data_file, "r").read().splitlines()
            unicode_transformer = UnicodeTransformer()
            words = [unicode_transformer.transform(word) for word in words]
            self.words_data = words
        return self.words_data



# -*- coding: utf-8 -*-
from unittest import TestCase
from underthesea import word_sent


class TestWord_sent(TestCase):
    def test_word_sent_1(self):
        sentence = u"cộng hòa xã hội chủ nghĩa"
        actual = word_sent(sentence)
        expected = u'cộng hòa xã_hội_chủ_nghĩa'
        self.assertEqual(actual, expected)

    def test_word_sent_2(self):
        sentence = u"hươu rất sợ tiếng động lạ ? ?"
        actual = word_sent(sentence)
        expected = u'hươu rất sợ tiếng_động lạ ? ?'
        self.assertEqual(actual, expected)

    def test_word_sent_3(self):
        sentence = u"Chúng ta thường nói đến Rau sạch, Rau an toàn để phân biệt với các rau bình thường bán ngoài chợ ."
        actual = word_sent(sentence)
        expected = u'Chúng_ta thường nói đến Rau_sạch , Rau an_toàn để phân_biệt với các rau bình_thường bán ngoài chợ .'
        self.assertEqual(actual, expected)

    def test_word_sent_4(self):
        sentence = u"Theo thông báo kết luận thanh tra của UBND tỉnh Thanh Hoá sáng nay 30/3, giai đoạn 2010-2015 Sở Xây dựng Thanh Hoá đã bổ nhiệm một số trưởng phòng, phó phòng chưa có trình độ Trung cấp lý luận chính trị, chưa qua lớp bồi dưỡng nghiệp vụ quản lý nhà nước, không đúng quy định của UBND tỉnh Thanh Hoá."
        actual = word_sent(sentence)
        expected = u'Theo thông_báo kết_luận thanh_tra của UBND tỉnh Thanh_Hoá sáng nay 30 / 3 , giai_đoạn 2010 - 2015 Sở_Xây_dựng Thanh_Hoá đã bổ_nhiệm một_số trưởng_phòng , phó_phòng chưa có trình_độ Trung_cấp lý_luận chính_trị , chưa qua lớp bồi_dưỡng nghiệp_vụ quản_lý nhà_nước , không đúng quy_định của UBND tỉnh Thanh_Hoá .'
        self.assertEqual(actual, expected)

    def test_word_sent_5(self):
        sentence = u"Tập thể lãnh đạo Sở Xây dựng không thực hiện nghiêm túc việc đánh giá toàn diện cán bộ trước khi đưa vào quy hoạch, tạo dư luận không tốt. Việc chưa báo cáo về Sở Nội vụ và không công khai việc bà Trần Vũ Quỳnh Anh thôi việc ngày 23/9/2016 thuộc trách nhiệm của Giám đốc Sở Xây dựng tỉnh Thanh Hoá."
        actual = word_sent(sentence)
        expected = u'Tập_thể lãnh_đạo Sở_Xây_dựng không thực_hiện nghiêm_túc việc đánh_giá toàn_diện cán_bộ trước khi đưa vào quy_hoạch , tạo dư_luận không tốt . Việc chưa báo_cáo về Sở_Nội_vụ và không công_khai việc bà Trần Vũ_Quỳnh Anh thôi_việc ngày 23 / 9 / 2016 thuộc trách_nhiệm của Giám_đốc Sở_Xây_dựng tỉnh Thanh_Hoá .'
        self.assertEqual(actual, expected)

    def test_word_sent_6(self):
        sentence = u"Có lẽ không ở đâu trên khắp thế giới bóng đá có giải vô địch quốc gia chịu chơi, chịu chi như giải nhà nghề Trung Quốc."
        actual = word_sent(sentence)
        expected = u'Có_lẽ không ở đâu trên khắp thế_giới bóng_đá có giải vô_địch_quốc_gia chịu_chơi , chịu chi như giải nhà_nghề Trung_Quốc .'
        self.assertEqual(actual, expected)

    def test_word_sent_7(self):
        sentence = u"Số những ngôi sao về chiều gia nhập giải nhà nghề Trung Quốc có Carlos Tevez (Argentina), Graziano Pelle (Italia), Obafemi Martins (Nigeria) ."
        actual = word_sent(sentence)
        expected = u'Số những ngôi_sao về chiều gia_nhập giải nhà_nghề Trung_Quốc có Carlos_Tevez ( Argentina ) , Graziano_Pelle ( Italia ) , Obafemi_Martins ( Nigeria ) .'
        self.assertEqual(actual, expected)

    def test_word_sent_8(self):
        sentence = u"Liên quan đến sự việc trên, sáng 30/3, trao đổi với phóng viên Dân trí, Thiếu tá Nguyễn Anh Dũng - Phó trưởng Công an TP Vĩnh Yên (Vĩnh Phúc) - cho biết, ngay khi nhận được thông tin, đơn vị này đã vào cuộc điều tra và xác định người đánh đập cháu H. là ông Hoàng Văn L. (bố đẻ của cháu H.)."
        actual = word_sent(sentence)
        expected = u'Liên_quan đến sự_việc trên , sáng 30 / 3 , trao_đổi với phóng_viên Dân_trí , Thiếu_tá Nguyễn_Anh_Dũng - Phó_trưởng Công_an TP Vĩnh_Yên ( Vĩnh_Phúc ) - cho_biết , ngay khi nhận được thông_tin , đơn_vị này đã vào cuộc điều_tra và xác_định người đánh_đập cháu H . là ông Hoàng_Văn_L . ( bố_đẻ của cháu H . ) .'
        self.assertEqual(actual, expected)

    def test_word_sent_9(self):
        sentence = u"Phát biểu trước báo giới, cựu tiền vệ MU, Paul Scholes đánh giá MU đứng trước cơ hội lớn vô địch Europa League"
        actual = word_sent(sentence)
        expected = u'Phát_biểu trước báo_giới , cựu tiền_vệ MU , Paul_Scholes đánh_giá MU đứng trước cơ_hội lớn vô_địch Europa_League'
        self.assertEqual(actual, expected)

    def test_word_sent_10(self):
        sentence = u"Hơn 2 tỷ USD vốn đầu tư này sẽ được huy động từ hai nguồn chính: Cụ thể, vốn góp của nhà đầu tư là Công ty CP Mặt Trời Vân Đồn (thuộc Sun Group) khoảng 7.125 tỷ đồng, tương đương 312,5 triệu USD, chiếm 15% tổng vốn đầu tư. 85% tổng vốn đầu tư còn lại (khoảng 40.356,9 tỷ đồng) sẽ vay từ Ngân hàng TMCP Công Thương Việt Nam (VietinBank) – chi nhánh Hà Nội theo tiến độ dự kiến từ quý II/2017 đến hết quý IV/2021."
        actual = word_sent(sentence)
        expected = u'Hơn 2 tỷ USD vốn đầu_tư này sẽ được huy_động từ hai nguồn chính : Cụ_thể , vốn góp của nhà_đầu_tư là Công_ty CP Mặt_Trời_Vân_Đồn ( thuộc Sun_Group ) khoảng 7.125 tỷ đồng , tương_đương 312,5 triệu USD , chiếm 15 % tổng vốn đầu_tư . 85 % tổng vốn đầu_tư còn lại ( khoảng 40.356,9 tỷ đồng ) sẽ vay từ Ngân_hàng TMCP_Công_Thương_Việt_Nam ( VietinBank ) – chi_nhánh Hà_Nội theo tiến_độ dự_kiến từ quý II / 2017 đến hết quý IV / 2021 .'
        self.assertEqual(actual, expected)

    def test_word_sent_11(self):
        sentence = u"Tích phân là một khái niệm toán học,và cùng với nghịch đảo của nó vi phân (differentiation) đóng vai trò là 2 phép tính cơ bản và chủ chốt trong lĩnh vực giải tích (calculus)."
        actual = word_sent(sentence)
        expected = u'Tích_phân là một khái_niệm toán_học , và cùng_với nghịch_đảo của nó vi_phân ( differentiation ) đóng vai_trò là 2 phép_tính cơ_bản và chủ_chốt trong lĩnh_vực giải_tích ( calculus ) .'
        self.assertEqual(actual, expected)

    def test_word_sent_12(self):
        sentence = u"Ý tưởng chủ đạo là tích phân và vi phân là hai phép tính nghịch đảo của nhau . "
        actual = word_sent(sentence)
        expected = u'Ý_tưởng chủ_đạo là tích_phân và vi_phân là hai phép_tính nghịch_đảo của nhau .'
        self.assertEqual(actual, expected)

    def test_word_sent_13(self):
        sentence = u"Vì ngày lang thang vẫn còn dài nên ta không kéo ga thật sâu"
        actual = word_sent(sentence)
        expected = u'Vì ngày lang_thang vẫn còn dài nên ta không kéo ga thật sâu'
        self.assertEqual(actual, expected)

    def test_word_sent_14(self):
        sentence = u"Vì vậy, để hiểu thêm về ngôn ngữ của người dân xứ Quảng qua đó hiểu về con người vùng đất này hơn và cũng để bổ sung thêm nguồn tư liệu dạy và học Ngữ văn địa phương, bài viết xin giới thiệu một số từ địa phương so với từ phổ thông (toàn dân) và một số âm địa phương so với âm phổ thông (chuẩn - toàn dân) của người Quảng Nam để các bạn đồng nghiệp và các em học sinh tham khảo"
        actual = word_sent(sentence)
        expected = u'Vì_vậy , để hiểu thêm về ngôn_ngữ của người dân xứ Quảng qua đó hiểu về con_người vùng_đất này hơn và cũng để bổ_sung thêm nguồn tư_liệu dạy và học Ngữ_văn địa_phương , bài viết xin giới_thiệu một_số từ địa_phương so với từ_phổ_thông ( toàn_dân ) và một_số âm địa_phương so với âm_phổ_thông ( chuẩn - toàn_dân ) của người Quảng_Nam để các bạn đồng_nghiệp và các em học_sinh tham_khảo'
        self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-
class Corpus:
    """Corpus is fundamental resource of NLP
    """
    def __init__(self):
        print("Corpus is fundamental resource of NLP")

    def load(self, folder):
        pass

    def save(self, folder):
        pass



# -*- coding: utf-8 -*-
from unittest import TestCase
from underthesea.transformer.lowercase import LowercaseTransformer


class TestUnicodeTransformer(TestCase):
    def test_transform_1(self):
        transformer = LowercaseTransformer()
        s = u"ĐI HỌC"
        output = transformer.transform(s)
        self.assertEqual(u"đi học", output)





# -*- coding: utf-8 -*-
from unittest import TestCase
from underthesea.word_sent.tokenize import tokenize


class TestWord_sent(TestCase):
    def test_tokenize(self):
        text = u"""Tổng thống Nga coi việc Mỹ không kích căn cứ quân sự của Syria là "sự gây hấn nhằm vào một quốc gia có chủ quyền", gây tổn hại đến quan hệ Moscow-Washington."""
        actual = tokenize(text)
        expected = u'Tổng thống Nga coi việc Mỹ không kích căn cứ quân sự của Syria là " sự gây hấn nhằm vào một quốc gia có chủ quyền " , gây tổn hại đến quan hệ Moscow - Washington .'
        self.assertEqual(actual, expected)

    def test_tokenize_2(self):
        text = u"""Vào tháng 10 năm ngoái, nghi phạm này từng bị bắt khi ăn cắp 25 mỹ phẩm trị giá 24.000 yen (hơn 200 USD) tại một cửa hàng giảm giá ở tỉnh Hyogo. Cùng năm, 4 thành viên khác cũng bị bắt vì ăn cắp một số lượng lớn son dưỡng môi và mỹ phẩm tại các cửa hiệu ở Osaka. Sau khi bị truy tố, 3 người trong số này được hưởng án treo."""
        actual = tokenize(text)
        expected = u'Vào tháng 10 năm ngoái , nghi phạm này từng bị bắt khi ăn cắp 25 mỹ phẩm trị giá 24.000 yen ( hơn 200 USD ) tại một cửa hàng giảm giá ở tỉnh Hyogo . Cùng năm , 4 thành viên khác cũng bị bắt vì ăn cắp một số lượng lớn son dưỡng môi và mỹ phẩm tại các cửa hiệu ở Osaka . Sau khi bị truy tố , 3 người trong số này được hưởng án treo .'
        self.assertEqual(actual, expected)

    def test_tokenize_3(self):
        text = u"""Giá khuyến mãi: 140.000đ / kg  ==> giảm được 20%"""
        actual = tokenize(text)
        expected = u'Giá khuyến mãi : 140.000 đ / kg ==> giảm được 20 %'
        self.assertEqual(actual, expected)

    def test_tokenize_4(self):
        text = u"""Microsoft hôm nay đã công bố cấu hình chính thức của thế hệ Xbox tiếp theo, tên mã Scorpio, và tin tưởng đây là loại console có sức mạnh xử lý cao nhất hiện giờ, mạnh hơn cả PS4 Pro. Tại buổi demo sản phẩm, Xbox Scorpio có thể chơi game đua xe Forza độ phân giải 4K với tốc độ đạt tới 60 khung hình/giây. Đồng thời Xbox Scorpio cũng sẽ tương thích với các tựa game Xbox cũ và quan trọng hơn nữa đó là các tựa game đó sẽ có đồ họa đẹp hơn, FPS cao hơn, thời gian load nhanh hơn khi chạy trên Xbox Scorpio."""
        actual = tokenize(text)
        expected = u'Microsoft hôm nay đã công bố cấu hình chính thức của thế hệ Xbox tiếp theo , tên mã Scorpio , và tin tưởng đây là loại console có sức mạnh xử lý cao nhất hiện giờ , mạnh hơn cả PS4 Pro . Tại buổi demo sản phẩm , Xbox Scorpio có thể chơi game đua xe Forza độ phân giải 4K với tốc độ đạt tới 60 khung hình / giây . Đồng thời Xbox Scorpio cũng sẽ tương thích với các tựa game Xbox cũ và quan trọng hơn nữa đó là các tựa game đó sẽ có đồ họa đẹp hơn , FPS cao hơn , thời gian load nhanh hơn khi chạy trên Xbox Scorpio .'
        self.assertEqual(actual, expected)

    def test_tokenize_5(self):
        text = u"""Tuyên bố của Điện Kremlin cũng nhấn mạnh, vụ tấn công sẽ “hủy hoại nghiêm trọng quan hệ Nga-Mỹ” và tạo ra “trở ngại cực lớn” cho việc tạo lập một liên minh quốc tế chống Tổ chức Nhà nước Hồi giáo tự xưng (IS)."""
        actual = tokenize(text)
        expected = u'Tuyên bố của Điện Kremlin cũng nhấn mạnh , vụ tấn công sẽ “ hủy hoại nghiêm trọng quan hệ Nga - Mỹ ” và tạo ra “ trở ngại cực lớn ” cho việc tạo lập một liên minh quốc tế chống Tổ chức Nhà nước Hồi giáo tự xưng ( IS ) .'
        self.assertEqual(actual, expected)

    def test_tokenize_6(self):
        text = u"""Ngày 11 tháng 3 là ngày thứ 70 (71 trong năm nhuận) trong lịch Gregory. Còn 295 ngày trong năm."""
        actual = tokenize(text)
        expected =u'Ngày 11 tháng 3 là ngày thứ 70 ( 71 trong năm nhuận ) trong lịch Gregory . Còn 295 ngày trong năm .'
        self.assertEqual(actual, expected)
